/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

static void __writer_internal (const char *, va_list, const char *);

void
sl_write (const char *fmt, ...)
{
  va_list ap;

  va_start (ap, fmt);
  __writer_internal (fmt, ap, NULL);
  va_end (ap);
}

void
sl_writeln (const char *fmt, ...)
{
  va_list ap;

  va_start (ap, fmt);
  __writer_internal (fmt, ap, NULL);
  va_end (ap);
  puts ("");
}

void
sl_writef (const char *file, const char *fmt, ...)
{
  va_list ap;

  va_start (ap, fmt);
  __writer_internal (fmt, ap, file);
  va_end (ap);
}

void
sl_writefln (const char *file, const char *fmt, ...)
{
  va_list ap;

  va_start (ap, fmt);
  __writer_internal (fmt, ap, file);
  va_end (ap);
  puts ("");
}

/* 1k, should be enough */
#define VSN_BUF_SIZE (1024)

static void
__writer_internal (const char *fmt, va_list ap, const char *filename)
{
  char buffer[VSN_BUF_SIZE];

  int c = vsnprintf (buffer, VSN_BUF_SIZE, fmt, ap);
  buffer[c] = '\0';

  if (filename != NULL)
    {
      FILE *file;

      if ((file = fopen (filename, "w")) != NULL)
	{
	  fputs (buffer, file);
	  fclose (file);
	}
      else
	fputs ("Failed to open log file.", stderr);
    }
  else
    fputs (buffer, stdout);

  fflush (NULL);
  return;
}

static struct precision
{
  int integer;
  int decimal;
}
precision;

void
sl_writer_set_precision (int d, int i)
{
  if (d < 0)
    d = -d;

  if (i < 0)
    i = -i;

  precision.decimal = d;
  precision.integer = i;
}

void
__precision_fmt (char *buf)
{
  char __buf[30];

  sprintf (__buf, "-%d.%dg", precision.integer, precision.decimal);

  strcpy (buf, "%");
  strcat (buf, __buf);
}

// TODO: use into the __writer_internal
void
sl_write_number (double n)
{
  char buf[40];
  char prec[40];

  __precision_fmt (prec);
  sprintf (buf, prec, n);
  puts (buf);
}
