/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#include <spar/sl_btree.h>
#include <spar/sl_sort.h>
#include <spar/sl_memory.h>

int
btdel (char *kin, sl_bt_node *hd)
{
  sl_bt_node *r, *s, **f;
  int ef;

  while (hd != NULL)
    {
      if ((ef = sl_strncmp (kin, hd->key, SL_IDENTIFIER_LENGHT)) == 0)
	break;
      else if (ef < 0)
	f = &(hd->pl);
      else
	f = &(hd->pr);
      hd = *f;
    }
  if (hd == NULL)
    return 0;

  if (hd->pr == NULL)
    *f = hd->pl;
  else if (hd->pl == NULL)
    *f = hd->pr;
  else if ((r = hd->pr)->pl == NULL)
    {
      r->pl = hd->pl;
      *f = r;
    }
  else
    {
      for (s = r->pl; s->pl != NULL;)
	{
	  r = s;
	  s = r->pl;
	}

      s->pl = hd->pl;
      r->pl = s->pr;
      s->pr = hd->pr;
      *f = s;
    }

  sl_free (hd, sizeof (hd));

  return 1;
}

sl_bt_node *
btins (char *kin, sl_bt_node *hd, char *data)
{
  sl_bt_node **v;
  int ef;

  while (hd != NULL)
    {
      if ((ef = sl_strncmp (kin, hd->key, SL_IDENTIFIER_LENGHT)) == 0)
	return hd;
      else if (ef < 0)
	v = &(hd->pl);
      else
	v = &(hd->pr);

      hd = *v;
    }

  hd = *v = (sl_bt_node *) sl_malloc (sizeof (*hd));
  strcpy(hd->key, kin);
  strcpy(hd->data, data);
  hd->pr = hd->pl = NULL;

  return hd;
}

sl_bt_node *
btsearch (char *kin, sl_bt_node *hd)
{
  int ef;

  while (hd != NULL)
    {
      if ((ef = sl_strncmp (kin, hd->key,SL_IDENTIFIER_LENGHT)) == 0)
	return hd;
      if (ef < 0)
	hd = hd->pl;
      else
	hd = hd->pr;
    }

  return hd;
}
