/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#ifndef _sl_function_h_
#define _sl_function_h_

#include <spar/sl_conf.h>

typedef enum
{
  SL_FUNCTION_WITH_ONE_ARG = 1,
  SL_FUNCTION_WITH_TWO_ARG,
  SL_FUNCTION_WITH_VAR_ARG,
  SL_FUNCTION_WITH_COUNTER
}
sl_function_args_t;

typedef enum
{
  SL_FUNCTION_CLASS_TRIG,
  SL_FUNCTION_CLASS_ITRIG,
  SL_FUNCTION_CLASS_EXP,
  SL_FUNCTION_CLASS_LOG,
  SL_FUNCTION_CLASS_HYPER,
  SL_FUNCTION_CLASS_SPECIAL,
  SL_FUNCTION_CLASS_USER,
  SL_FUNCTION_CLASS_COUNTER
}
sl_function_class_t;

typedef double (*sl_function_p) (double);
typedef double (*sl_function_tp) (double, double);

struct sl_function_s
{
  sl_identifier (name);
  sl_message (math);
  sl_function_args_t args;
  sl_function_class_t type;
  sl_function_p func;
  sl_function_tp func2;
};

typedef struct sl_function_s sl_function;

struct sl_function_help_s
{
  sl_identifier (name);
  sl_message (math);
  sl_message (type);
  sl_function_args_t args;
};

typedef struct sl_function_help_s sl_function_help;

__BEGIN_DECLS 

int sl_function_init (void);
double sl_function_run (const char *name, double argv[]);
sl_function_args_t sl_function_get_argc (int);
const char *sl_function_get_name (int code);
int sl_function_check (const char *name);
void sl_function_print_all (void);
int sl_function_make_help (const char *name, sl_function_help * tmp);

__END_DECLS
#endif /* _funcs_h_ */
