/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#include <spar/sl_variable.h>
#include <spar/sl_module_manager.h>
#include <spar/sl_conf.h>
#include <spar/sl_memory.h>
#include <spar/sl_constant.h>
#include <spar/sl_function.h>

static int __is_already_init = 0;

int
sl_init (void)
{
  if (__is_already_init == 1)
    return SL_ERROR;

  sl_variable_init ();
  sl_module_manager_init ();
  sl_function_init ();
  sl_constant_init ();

/*
 *  Set the init flag to 1
 *  this means that the this function was already called
 */
  __is_already_init = 1;

  return SL_SUCCESS;
}

int
sl_de_init (void)
{
  if (__is_already_init == 1)
    {
      sl_variable_remove_all ();
      sl_module_manager_remove_all ();
#ifdef DEBUG
      sl_memory_stats ();
#endif
      __is_already_init = 0;
      return SL_SUCCESS;
    }
  else
    return SL_ERROR;
}
