/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <math.h>
#include <spar/sl_math_library.h>

SL_MATH_FUNCTION (sl_acosh)
#ifndef HAVE_ACOSH
{
    if (x > 1.0 / SL_DOUBLE_EPSILON)
      {
	return sl_log (x) + M_LN2;
      }
    else if (x > 2)
      {
	return sl_log (2 * x - 1 / (sl_sqrt (sl_pow_2 (x) - 1) + x));
      }
    else if (x > 1)
      {
	double t = x - 1;
	return sl_log1p (t + sl_sqrt (2 * t + sl_pow_2 (x)));
      }
    else if (x == 1)
      {
	return 0;
      }
    else
      {
	return SL_NAN;
      }
  }
#else
{
  return acosh (x);
}
#endif
