/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <math.h>
#include <spar/sl_math_library.h>
#include <spar/sl_math.h>

SL_MATH_FUNCTION (sl_asinh)
#ifndef HAVE_ASINH
{
  double a = fabs (x);
  double s = (x < 0) ? -1 : 1;

  if (a > 1 / SL_DOUBLE_EPSILON)
    {
      return s * (sl_log (a) + M_LN2);
    }
  else if (a > 2)
    {
      return s * sl_log (2 * a + 1 / (a + sl_sqrt (sl_pow_2(a) + 1)));
    }
  else if (a > SL_DOUBLE_EPSILON)
    {
      double a2 = sl_pow_2(a);
      return s * sl_log1p (a + a2 / (1 + sl_sqrt (1 + a2)));
    }
  else
    {
      return x;
    }
}
#else
{
  return asinh (x);
}
#endif
