/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <math.h>
#include <spar/sl_math_library.h>

SL_MATH_FUNCTION (sl_atanh)
#ifndef HAVE_ATANH
{
  double a = fabs (x);
  double s = (x < 0) ? -1 : 1;

  if (a > 1)
    {
      return SL_NAN;
    }
  else if (a == 1)
    {
      return (x < 0) ? SL_NEGINF : SL_POSINF;
    }
  else if (a >= 0.5)
    {
      return s * 0.5 * sl_log1p (2 * a / (1 - a));
    }
  else if (a > SL_DOUBLE_EPSILON)
    {
      return s * 0.5 * sl_log1p (2 * a + 2 * a * a / (1 - a));
    }
  else
    {
      return x;
    }
}
#else
{
  return atanh (x);
}
#endif
