/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#ifndef _sl_math_h_
#define _sl_math_h_

#undef M_E
#undef M_LOG2E
#undef M_LOG10E
#undef M_LN2
#undef M_LN10
#undef M_PI
#undef M_PI_2
#undef M_PI_4
#undef M_1_PI
#undef M_2_PI
#undef M_2_SQRTPI
#undef M_SQRT2
#undef M_SQRT1_2

#define M_E            2.7182818284590452354	/* e */
#define M_LOG2E        1.4426950408889634074	/* log_2 e */
#define M_LOG10E       0.43429448190325182765	/* log_10 e */
#define M_LN2          0.69314718055994530942	/* log_e 2 */
#define M_LN10         2.30258509299404568402	/* log_e 10 */
#define M_PI           3.14159265358979323846	/* pi */
#define M_PI_2         1.57079632679489661923	/* pi/2 */
#define M_PI_4         0.78539816339744830962	/* pi/4 */
#define M_1_PI         0.31830988618379067154	/* 1/pi */
#define M_2_PI         0.63661977236758134308	/* 2/pi */
#define M_2_SQRTPI     1.12837916709551257390	/* 2/sqrt(pi) */
#define M_SQRT2        1.41421356237309504880	/* sqrt(2) */
#define M_SQRT1_2      0.70710678118654752440	/* 1/sqrt(2) */

/*
 *  Some other constants
 */
#define SL_NAN                  (0.0 / 0.0)
#define SL_POSINF               (+1.0 / 0.0)
#define SL_NEGINF               (-1.0 / 0.0)
#define ZERO 0.0

/*
 *  Internal use
 */
#define __SL_MAX(a,b)      ((a) > (b) ? (a) : (b))
#define __SL_MIN(a,b)      ((a) > (b) ? (b) : (a))

#ifdef HAVE_INLINE
extern inline double
SL_MAX (double a, double b)
{
  return __SL_MAX (a, b);
}

extern inline double
SL_MIN (double a, double b)
{
  return __SL_MIN (a, b);
}
#else /* warning: this don't checks the types */
#define SL_MAX(a,b)   __SL_MAX(a,b)
#define SL_MIN(a,b)   __SL_MIN(a,b)
#endif /* HAVE_INLINE */


#define SL_IS_ODD(x)     ((x) &1)
#define SL_IS_EVEN(x)    (!(SL_IS_ODD(x)))
#define SL_ABS(x)        (((x) < ZERO) ? (-(x)) : (x))
#define SL_SGN(x)        ((x) > ZERO? 1 : -1)
#define SL_IS_REAL(x) 	 (finite(x))

#define SL_IS_ZERO(x) 		(x == ZERO)
#define SL_IS_NOT_ZERO(x)	(x != ZERO)
#define SL_IS_NEGATIVE(x) 		(x < ZERO)
#define SL_IS_NOT_NEGATIVE(x) 		(!(SL_IS_NEGATIVE(x)))
#define SL_IS_POSITIVE(x)		(x > ZERO)
#define SL_IS_NOT_POSITIVE(x)		(!(SL_IS_NOT_POSITIVE(x)))

#define SL_TO_RAD(x) ((x * M_PI) / 180.0)
#define SL_TO_DEG(x) ((x * 180.0) / M_PI)

#define SL_DBL_SWAP(a,b) do { double tmp = b ; b = a ; a = tmp ; } while(0)

/*
 *  Some other constants (gsl-0.8 source)
 */
#define SL_DOUBLE_EPSILON    2.2204460492503131e-16
#define SL_DOUBLE_MIN        2.2250738585072014e-308
#define SL_DOUBLE_MAX        1.7976931348623157e+308
#endif
