/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#ifndef _math_library_h_
#define _math_library_h_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <spar/sl_conf.h>

#define SL_MATH_FUNCTION(name) double name(const double x)
#define SL_MATH_FUNCTION_2(name) double name(const double x, const double y)

__BEGIN_DECLS
/* one arg functions */
  SL_MATH_FUNCTION (sl_deg);
SL_MATH_FUNCTION (sl_rad);
SL_MATH_FUNCTION (sl_arcsin);
SL_MATH_FUNCTION (sl_arccos);
SL_MATH_FUNCTION (sl_arctan);
SL_MATH_FUNCTION (sl_cos);
SL_MATH_FUNCTION (sl_sin);
SL_MATH_FUNCTION (sl_tan);
SL_MATH_FUNCTION (sl_sinh);
SL_MATH_FUNCTION (sl_cosh);
SL_MATH_FUNCTION (sl_tanh);
SL_MATH_FUNCTION (sl_log);
SL_MATH_FUNCTION (sl_log2);
SL_MATH_FUNCTION (sl_log10);
SL_MATH_FUNCTION (sl_exp);
SL_MATH_FUNCTION (sl_abs);
SL_MATH_FUNCTION (sl_inv);
SL_MATH_FUNCTION (sl_floor);
SL_MATH_FUNCTION (sl_ceil);
SL_MATH_FUNCTION (sl_fat);
SL_MATH_FUNCTION (sl_sum);
SL_MATH_FUNCTION (sl_sgn);


SL_MATH_FUNCTION_2 (sl_mcd);
SL_MATH_FUNCTION_2 (sl_expn);
SL_MATH_FUNCTION_2 (sl_sqrn);
SL_MATH_FUNCTION_2 (sl_max);
SL_MATH_FUNCTION_2 (sl_min);
SL_MATH_FUNCTION_2 (sl_pow);
SL_MATH_FUNCTION_2 (sl_mod);

#ifdef HAVE_INLINE
extern inline SL_MATH_FUNCTION (sl_pow_1);
extern inline SL_MATH_FUNCTION (sl_pow_2);
extern inline SL_MATH_FUNCTION (sl_pow_3);
extern inline SL_MATH_FUNCTION (sl_pow_4);
extern inline SL_MATH_FUNCTION (sl_pow_5);
extern inline SL_MATH_FUNCTION (sl_pow_6);
extern inline SL_MATH_FUNCTION (sl_pow_7);
extern inline SL_MATH_FUNCTION (sl_pow_8);
extern inline SL_MATH_FUNCTION (sl_pow_9);

inline SL_MATH_FUNCTION (sl_pow_1) { return x; }
inline SL_MATH_FUNCTION (sl_pow_2) { return sl_pow_1 (x) * sl_pow_1 (x); }
inline SL_MATH_FUNCTION (sl_pow_3) { return sl_pow_2 (x) * sl_pow_1 (x); }
inline SL_MATH_FUNCTION (sl_pow_4) { return sl_pow_2 (x) * sl_pow_2 (x); }
inline SL_MATH_FUNCTION (sl_pow_5) { return sl_pow_3 (x) * sl_pow_2 (x); }
inline SL_MATH_FUNCTION (sl_pow_6) { return sl_pow_3 (x) * sl_pow_3 (x); }
inline SL_MATH_FUNCTION (sl_pow_7) { return sl_pow_4 (x) * sl_pow_3 (x); }
inline SL_MATH_FUNCTION (sl_pow_8) { return sl_pow_4 (x) * sl_pow_4 (x); }
inline SL_MATH_FUNCTION (sl_pow_9) { return sl_pow_5 (x) * sl_pow_4 (x); }

#else
SL_MATH_FUNCTION (sl_pow_2);
SL_MATH_FUNCTION (sl_pow_3);
SL_MATH_FUNCTION (sl_pow_4);
SL_MATH_FUNCTION (sl_pow_5);
SL_MATH_FUNCTION (sl_pow_6);
SL_MATH_FUNCTION (sl_pow_7);
SL_MATH_FUNCTION (sl_pow_8);
SL_MATH_FUNCTION (sl_pow_9);
#endif

/*
 *  There is avaiable a "Spar" version
 *  of the following functions (see config.h)
 *
 */
SL_MATH_FUNCTION (sl_isnan);
SL_MATH_FUNCTION (sl_isinf);
SL_MATH_FUNCTION (sl_finite);
SL_MATH_FUNCTION (sl_expm1);
SL_MATH_FUNCTION (sl_log1p);
SL_MATH_FUNCTION (sl_acosh);
SL_MATH_FUNCTION (sl_asinh);
SL_MATH_FUNCTION (sl_atanh);

SL_MATH_FUNCTION_2 (sl_hypot);

__END_DECLS

#endif /* _sl_math_library_h_ */
