/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

/**
 *  Parser interface
 *
 *  This file contains all definitions to deal with the parser, however is
 *  still incomplete.
 *
 *  todo: create a function that inits the parser (token init, error init...)
 *  todo: add a function that print the result to a string
 */

#ifndef _sl_parser_h_
#define _sl_parser_h_

#include <spar/sl_error.h>
#include <spar/sl_conf.h>

__BEGIN_DECLS
/**
 *  Evaluate the string str
 * 
 *  This function returns directly the string value and should be used 
 *  is the most of cases. If there are any error, returns an intermediate
 *  result and prints an error message (@see parser_throw_error())
 */
double sl_parser_evaluate_to_double (const char *str);

/**
 *  Evaluate the string str
 *
 *  Evaluate a string and returns the error code (@see parser_error_t)
 *  If there are any error, returns an intermediate
 *  result and prints an error message (@see parser_throw_error())
 */
sl_error sl_parser_evaluate (const char *str);

__END_DECLS
#endif /* _parser_h_ */
