/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */

#include <spar/sl_sort.h>

#define SWAP(x,y) (tmp=(x), (x)=(y), (y)=tmp)

void
rheap (void **v, int k, int n, int (*cmp) ())
{
  int m = n - 1, j;
  char *tmp;

  for (j = 2 * k + 1; j < n; k = j, j = 2 * k + 1)
    {
      if (j < m && (*cmp) (v[j + 1], v[j]) > 0)
	++j;

      if ((*cmp) (v[j], v[k]) > 0)
	SWAP (v[k], v[j]);
      else
	break;
    }
}

int
sl_hsort (void **v, int n, int (*cmp) ())
{
  int k;
  char *tmp;

  for (k = n / 2 - 1; k >= 0;)
    rheap (v, k--, n, cmp);

  for (--n; n > 0;)
    {
      SWAP (v[0], v[n]);
      rheap (v, 0, n--, cmp);
    }

  return SL_SUCCESS;
}
