/*
 * The Spar Library - modular math parser
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 *
 */
#include <stdlib.h>
#include <spar/sl_writer.h>
#include <spar/sl_conf.h>
#include <spar/sl_error.h>

#ifdef DEBUG
int __allocated;
int __de_allocated;
int __free_counter;
int __malloc_counter;
#endif

void
sl_memory_init (void)
{
#ifdef DEBUG
  __allocated = 0;
  __de_allocated = 0;
  __free_counter = 0;
  __malloc_counter = 0;
#endif
}

void *
sl_malloc (int bytes)
{
  void *m = malloc (bytes);

  if (m == NULL)
    sl_error_throw (SL_ERROR_CLASS_FATAL, SL_ERROR_NO_MEM, "(%u).", bytes);

#ifdef DEBUG
  __allocated += bytes;
  __malloc_counter++;
#endif

  return m;
}

/* TODO: add NULL check */
void
sl_free (void *m, int bytes)
{
#ifdef DEBUG
  __de_allocated += bytes;
  __free_counter++;
#endif

  if (m == NULL)
    sl_error_throw (SL_ERROR_CLASS_FATAL, SL_ERROR_FREE_TWICE, "");

  free (m);
}

void
sl_memory_stats (void)
{
#ifdef DEBUG
  // TODO: stats in Kb
  sl_writeln ("Memory Stats");
  sl_writeln ("used   : %d bytes (%d calls).", __allocated, __malloc_counter);
  sl_writeln ("frezed : %d bytes (%d calls).", __de_allocated,
	      __free_counter);
#endif
}
