/*
 * Spar - a text frontend for the Spar library 
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include <string.h>
#include <spar/sl.h>

#include "commands.h"
#include "commands_table.h"
#include "prefs.h"

extern void clean_exit (int exit_code);

COMMAND (cmd_exit)
{
  clean_exit (0);
}

COMMAND (cmd_clr_all)
{
  sl_variable_remove_all ();
}

COMMAND (cmd_warranty)
{
  sl_writeln ("%s%s%s%s%s%s%s%s%s%s%s",
	      "\t This program is free software; you can redistribute it and/or modify\n",
	      "\t it under the terms of the GNU General Public License as published by\n",
	      "\t the Free Software Foundation; either version 2 of the License , or\n",
	      "\t (at your option) any later version.\n\n",
	      "\t This program is distributed in the hope that it will be useful,\n",
	      "\t but WITHOUT ANY WARRANTY; without even the implied warranty of\n",
	      "\t MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n",
	      "\t GNU General Public License for more details.\n\n",
	      "\t You should have received a copy of the GNU General Public License\n",
	      "\t along with this program. If not, write to the Free Software\n",
	      "\t Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.");
}

COMMAND (cmd_set_old_value)
{
  if (argv[1] == NULL)
    sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_CUSTOM,
		    "%s command gets at least one arguments.", argv[0]);
  sl_variable_old (argv[1]);
}


COMMAND (cmd_help)
{
  print_cmd_table ();
}

COMMAND (cmd_add_var_comment)
{
  if (argc < 3)
    {
      sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_CUSTOM,
		      "this command gets at least 2 arguments. \n"
		      "Type 'help %s' for the command's syntax.", argv[0]);

      return;
    }

  else
    {
      sl_message (comment) = " ";
      register int i, comment_length = 0;
      for (i = 2; i < argc; i++)
	comment_length = strlen (argv[i]) + 1;

      if (comment_length > SL_MESSAGE_LENGHT)
	{
	  sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_MESSAGE, "");
	  return;
	}

      for (i = 2; i < argc; i++)
	{
	  strcat (comment, argv[i]);
	  strcat (comment, " ");
	}

      if ((sl_variable_comment (argv[1], comment)))
	{
	  sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_MESSAGE,
			  "failed to comment the variable '%s'.", argv[1]);
	  return;
	}
    }
  return;
}

COMMAND (cmd_consts)
{
  sl_constant_print_all ();
}

COMMAND (cmd_vars)
{
  sl_variable_print_all ();
}

COMMAND (cmd_funcs)
{
  switch (argc)
    {
    case 1:
      {
	sl_function_print_all ();
	break;
      }
    case 2:
      {
	sl_function_help doc;
	if (sl_function_make_help (argv[1], &doc))
	  {
	    sl_writeln ("Function \"%s\":", doc.name);
	    sl_writeln ("\t[arguments] :\t%u", doc.args);
	    sl_writeln ("\t[math form] :\t%s", doc.math);
	    sl_writeln ("\t[math type] :\t%s", doc.type);
	  }
	else
	  sl_error_throw (SL_ERROR_CLASS_ERROR, SL_ERROR_MESSAGE,
			  "unknown function '%s'.", argv[1]);
	break;
      }
    default:
      break;
    }
  return;
}


COMMAND (cmd_clr)
{
  register int i, rc;

  for (i = 1; i < argc; i++)
    {
      sl_write ("deleting variable '%s'...", argv[i]);
      rc = !sl_variable_remove (argv[i]);
      rc ? sl_writeln ("cleared.") : sl_writeln ("unknown.");
    }
}

/*
 * The following commands implements the module support of this program.
 * There are 3 basic actions that you can do with modules:
 *     running      run a module   
 *     showing      show infos about a module
 */
COMMAND (cmd_mod_run)
{
  if (argv[1] == NULL) {
        sl_writeln ("No module specified. Please pick a module from this list.");	
	sl_module_manager_print_all ();

		
  }
 else
  sl_module_manager_run (argv[1]);
}

COMMAND (cmd_mod_show)
{
  switch (argc)
    {
    case 1:
      sl_module_manager_print_all ();
      break;
    case 2:
      sl_module_manager_print (argv[1]);
      break;
    default:
      break;
    }
}
