/*
 * Spar - a text frontend for the Spar library
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
#include <string.h>
#include <spar/sl_conf.h>
#include <spar/sl_writer.h>
#include <spar/sl_sort.h>
#include <spar/sl_table.h>

#include "commands_table.h"
#include "prefs.h"

struct cmd_entry Commands[] = {
  {
   "quit",			/* complete command                     */
   "",				/* usage message                        */
   "return to the shell",	/* command help                         */
   cmd_exit			/* pointer to function                  */
   },

  {
   "clrs",
   "",
   "delete all variables",
   cmd_clr_all},

  {
   "warranty",
   "",
   "print the GNU license",
   cmd_warranty},

  {
   "vars",
   "[variable]",
   "printout the vars-table",
   cmd_vars},

  {
   "consts",
   "[constant]",
   "print the constants table",
   cmd_consts},

  {
   "funcs",
   "[function]",
   "print the list of the supported functions",
   cmd_funcs},

  {
   "help",
   "[command]",
   "show commands",
   cmd_help},

  {
   "delete",
   "[variable, variable, ...]",
   "delete a list of variables",
   cmd_clr},

  {
   "comment",
   "[variable], [comment, comment, ...]",
   "add a comment to the variable",
   cmd_add_var_comment},

  {
   "old",
   "[variable]",
   "set the last value as new value",
   cmd_set_old_value},

  {
   "run",
   "[module]",
   "run a module",
   cmd_mod_run},

  {
   "modules",
   "[module]",
   "list only the loaded modules",
   cmd_mod_show}
};

const int COMMANDS_COUNTER = sizeof (Commands) / sizeof (Commands[0]);

int
__command_cmp (const struct cmd_entry *c1, const struct cmd_entry *c2)
{
  return sl_strncmp (c1->cmd, c2->cmd, SL_IDENTIFIER_LENGHT);
}

int
init_commands (void)
{
  return sl_qsort (Commands,
		   COMMANDS_COUNTER,
		   sizeof (struct cmd_entry),
		   (int (*)(const void *, const void *)) __command_cmp);
}

int
check_command (const char str[])
{
  if (str == NULL)
    return SL_SYMBOL_NOT_FOUND;
  else
    {
      struct cmd_entry *r;

      r = sl_bsearch (str,
		      Commands,
		      COMMANDS_COUNTER,
		      sizeof (struct cmd_entry),
		      (int (*)(const void *, const void *)) __command_cmp);
      if (r)
	return ((int) r - (int) Commands) / sizeof (struct cmd_entry);
      else
	return SL_SYMBOL_NOT_FOUND;
    }
}

void
__print_command (int index)
{
  sl_write ("%-*s", SL_IDENTIFIER_LENGHT, Commands[index].cmd);
  //sl_write ("%-*s", 25, Commands[index].usage);
  sl_write ("%-*s", SL_MESSAGE_LENGHT-1, Commands[index].help);
}

void
print_cmd_table (void)
{
  sl_table_column command_cols[] = {
    {"Name", SL_IDENTIFIER_LENGHT},
    //{"Arg", 25},
    {"Use", SL_MESSAGE_LENGHT-1},
    {{0}}
  };

  sl_table_create ("Commands",
                command_cols,
                COMMANDS_COUNTER,
                __print_command);
}
