/*
 * Spar - a text frontend for the Spar library
 * Copyright (C) 2000,2001 Davide Angelocola <davide178@inwind.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <signal.h>
#include "terminal.h"

extern void clean_exit (int);

static void
install_handler (void)
{
  struct sigaction setup_action;
  sigset_t block_mask;

  /* set block mask */
  sigemptyset (&block_mask);
  sigaddset (&block_mask, SIGINT);
  sigaddset (&block_mask, SIGQUIT);

  /* set action handler */
  setup_action.sa_handler = clean_exit;
  setup_action.sa_mask = block_mask;
  setup_action.sa_flags = 0;
  sigaction (SIGTSTP, &setup_action, 0);

  return;
}

static struct termios old;
static struct termios new_ts;

void
restore_term (void)
{
  tcsetattr (fileno (stdin), TCSANOW, &old);
}

void
init_term (void)
{
  install_handler ();
  signal (SIGINT, clean_exit);
  signal (SIGQUIT, clean_exit);
  signal (SIGKILL, clean_exit);

  tcgetattr (fileno (stdin), &old);
  new_ts = old;
  new_ts.c_cc[VEOF] = 1;	/* don't allow CTRL-D */

  if ((tcsetattr (fileno (stdin), TCSAFLUSH, &new_ts)) < 0)
    {
      restore_term ();
      exit (-1);
    }

  return;
}
